@@ -17,14 +17,17 @@
 
 from __future__ import absolute_import, division, print_function
 
+import linecache
 import logging
 import os
 from io import open
 from multiprocessing import Pool, cpu_count
 
-from tqdm.auto import tqdm
-
 import pandas as pd
+import torch
+from torch.nn import CrossEntropyLoss
+from torch.utils.data import Dataset
+from tqdm.auto import tqdm
 
 
 class InputExample(object):
@@ -269,3 +272,72 @@ def get_labels(path):
             "B-LOC",
             "I-LOC",
         ]
+
+
+class LazyNERDataset(Dataset):
+    def __init__(self, data_file, tokenizer, args):
+        self.data_file = data_file
+        self.lazy_loading_start_line = args.lazy_loading_start_line if args.lazy_loading_start_line else 0
+        self.example_lines, self.num_entries = self._get_examples(self.data_file, self.lazy_loading_start_line)
+        self.tokenizer = tokenizer
+        self.args = args
+        self.pad_token_label_id = CrossEntropyLoss().ignore_index
+
+    @staticmethod
+    def _get_examples(data_file, lazy_loading_start_line):
+        example_lines = {}
+        start = lazy_loading_start_line
+        entry_num = 0
+        with open(data_file, encoding="utf-8") as f:
+            for line_idx, _ in enumerate(f, 1):
+                if _ == "\n" and line_idx > lazy_loading_start_line:
+                    example_lines[entry_num] = (start, line_idx)
+                    start = line_idx + 1
+                    entry_num += 1
+
+        return example_lines, entry_num
+
+    def __getitem__(self, idx):
+        start, end = self.example_lines[idx]
+        words, labels = [], []
+        for idx in range(start, end):
+            line = linecache.getline(self.data_file, idx).rstrip("\n")
+            splits = line.split(" ")
+            words.append(splits[0])
+            if len(splits) > 1:
+                labels.append(splits[-1].replace("\n", ""))
+            else:
+                # Examples could have no label for mode = "test"
+                labels.append("O")
+        if words:
+            example = InputExample(guid="%s-%d".format("train", idx), words=words, labels=labels)
+
+        label_map = {label: i for i, label in enumerate(self.args.labels_list)}
+
+        example_row = (
+            example,
+            label_map,
+            self.args.max_seq_length,
+            self.tokenizer,
+            bool(self.args.model_type in ["xlnet"]),
+            self.tokenizer.cls_token,
+            2 if self.args.model_type in ["xlnet"] else 0,
+            self.tokenizer.sep_token,
+            bool(self.args.model_type in ["roberta"]),
+            bool(self.args.model_type in ["xlnet"]),
+            self.tokenizer.convert_tokens_to_ids([self.tokenizer.pad_token])[0],
+            4 if self.args.model_type in ["xlnet"] else 0,
+            self.pad_token_label_id,
+            0,
+            True,
+        )
+
+        features = convert_example_to_feature(example_row)
+        all_input_ids = torch.tensor(features.input_ids, dtype=torch.long)
+        all_input_mask = torch.tensor(features.input_mask, dtype=torch.long)
+        all_segment_ids = torch.tensor(features.segment_ids, dtype=torch.long)
+        all_label_ids = torch.tensor(features.label_ids, dtype=torch.long)
+        return (all_input_ids, all_input_mask, all_segment_ids, all_label_ids)
+
+    def __len__(self):
+        return self.num_entries
