@@ -4,31 +4,30 @@ import math
 import os
 import random
 import warnings
-from multiprocessing import cpu_count, Pool
-from pathlib import Path
 from dataclasses import asdict
+from multiprocessing import Pool, cpu_count
+from pathlib import Path
 
 import numpy as np
-from tqdm.auto import tqdm, trange
-
 import pandas as pd
 import torch
-from simpletransformers.config.global_args import global_args
-from simpletransformers.config.model_args import Seq2SeqArgs
-from simpletransformers.seq2seq.seq2seq_utils import Seq2SeqDataset, SimpleSummarizationDataset
 from tensorboardX import SummaryWriter
 from torch.nn.utils.rnn import pad_sequence
 from torch.utils.data import DataLoader, Dataset, RandomSampler, SequentialSampler
 from torch.utils.data.distributed import DistributedSampler
-from transformers import AdamW, EncoderDecoderModel, EncoderDecoderConfig, get_linear_schedule_with_warmup
+from tqdm.auto import tqdm, trange
 from transformers import (
+    AdamW,
+    AutoConfig,
     AutoModel,
     AutoTokenizer,
-    AutoConfig,
-    BertTokenizer,
-    BertModel,
-    BertForMaskedLM,
+    BartConfig,
+    BartForConditionalGeneration,
+    BartTokenizer,
     BertConfig,
+    BertForMaskedLM,
+    BertModel,
+    BertTokenizer,
     CamembertConfig,
     CamembertModel,
     CamembertTokenizer,
@@ -38,9 +37,14 @@ from transformers import (
     ElectraConfig,
     ElectraModel,
     ElectraTokenizer,
+    EncoderDecoderConfig,
+    EncoderDecoderModel,
     LongformerConfig,
     LongformerModel,
     LongformerTokenizer,
+    MarianConfig,
+    MarianMTModel,
+    MarianTokenizer,
     MobileBertConfig,
     MobileBertModel,
     MobileBertTokenizer,
@@ -49,14 +53,13 @@ from transformers import (
     RobertaConfig,
     RobertaModel,
     RobertaTokenizer,
-    BartForConditionalGeneration,
-    BartTokenizer,
-    BartConfig,
-    MarianMTModel,
-    MarianTokenizer,
-    MarianConfig,
+    get_linear_schedule_with_warmup,
 )
 
+from simpletransformers.config.global_args import global_args
+from simpletransformers.config.model_args import Seq2SeqArgs
+from simpletransformers.seq2seq.seq2seq_utils import Seq2SeqDataset, SimpleSummarizationDataset
+
 try:
     import wandb
 
@@ -104,7 +107,7 @@ class Seq2SeqModel:
             decoder_name (optional): The exact architecture and trained weights to use. This may be a Hugging Face Transformers compatible pre-trained model, a community model, or the path to a directory containing model files.
                                     Must be the same "size" as the encoder model (base/base, large/large, etc.)
             encoder_decoder_type (optional): The type of encoder-decoder model. (E.g. bart)
-            encoder_decoder_name (optional): The path to a directory containing the saved encoder and decoder of a Seq2SeqModel. (E.g. "outputs/") OR a valid BART model.
+            encoder_decoder_name (optional): The path to a directory containing the saved encoder and decoder of a Seq2SeqModel. (E.g. "outputs/") OR a valid BART or MarianMT model.
             config (optional): A configuration file to build an EncoderDecoderModel.
             args (optional): Default args will be used if this parameter is not provided. If provided, it should be a dict containing the args that should be changed in the default args.
             use_cuda (optional): Use GPU if available. Setting to False will force model to use CPU only.
@@ -234,7 +237,7 @@ class Seq2SeqModel:
         Args:
             train_data: Pandas DataFrame containing the 2 columns - `input_text`, `target_text`.
                         - `input_text`: The input text sequence.
-                        - `target_text`: The target sequence
+                        - `target_text`: The target text sequence
             output_dir: The directory where model files will be saved. If not given, self.args.output_dir will be used.
             show_running_loss (optional): Set to False to prevent running loss from being printed to console. Defaults to True.
             args (optional): Optional changes to the args dict of the model. Any changes made will persist for the model.
@@ -308,7 +311,12 @@ class Seq2SeqModel:
 
         tb_writer = SummaryWriter(logdir=args.tensorboard_dir)
         train_sampler = RandomSampler(train_dataset)
-        train_dataloader = DataLoader(train_dataset, sampler=train_sampler, batch_size=args.train_batch_size)
+        train_dataloader = DataLoader(
+            train_dataset,
+            sampler=train_sampler,
+            batch_size=args.train_batch_size,
+            num_workers=self.args.dataloader_num_workers,
+        )
 
         if args.max_steps > 0:
             t_total = args.max_steps
@@ -388,14 +396,6 @@ class Seq2SeqModel:
             optimizer.load_state_dict(torch.load(os.path.join(args.model_name, "optimizer.pt")))
             scheduler.load_state_dict(torch.load(os.path.join(args.model_name, "scheduler.pt")))
 
-        if args.fp16:
-            try:
-                from apex import amp
-            except ImportError:
-                raise ImportError("Please install apex from https://www.github.com/nvidia/apex to use fp16 training.")
-
-            model, optimizer = amp.initialize(model, optimizer, opt_level=args.fp16_opt_level)
-
         if args.n_gpu > 1:
             model = torch.nn.DataParallel(model)
 
@@ -439,6 +439,11 @@ class Seq2SeqModel:
             wandb.init(project=args.wandb_project, config={**asdict(args)}, **args.wandb_kwargs)
             wandb.watch(self.model)
 
+        if args.fp16:
+            from torch.cuda import amp
+
+            scaler = amp.GradScaler()
+
         model.train()
         for current_epoch in train_iterator:
             if epochs_trained > 0:
@@ -458,9 +463,15 @@ class Seq2SeqModel:
                 # batch = tuple(t.to(device) for t in batch)
 
                 inputs = self._get_inputs_dict(batch)
-                outputs = model(**inputs)
-                # model outputs are always tuple in pytorch-transformers (see doc)
-                loss = outputs[0]
+                if args.fp16:
+                    with amp.autocast():
+                        outputs = model(**inputs)
+                        # model outputs are always tuple in pytorch-transformers (see doc)
+                        loss = outputs[0]
+                else:
+                    outputs = model(**inputs)
+                    # model outputs are always tuple in pytorch-transformers (see doc)
+                    loss = outputs[0]
 
                 if args.n_gpu > 1:
                     loss = loss.mean()  # mean() to average on multi-gpu parallel training
@@ -476,25 +487,21 @@ class Seq2SeqModel:
                     loss = loss / args.gradient_accumulation_steps
 
                 if args.fp16:
-                    with amp.scale_loss(loss, optimizer) as scaled_loss:
-                        scaled_loss.backward()
-                    # torch.nn.utils.clip_grad_norm_(
-                    #     amp.master_params(optimizer), args.max_grad_norm
-                    # )
+                    scaler.scale(loss).backward()
                 else:
                     loss.backward()
-                    # torch.nn.utils.clip_grad_norm_(
-                    #     model.parameters(), args.max_grad_norm
-                    # )
 
                 tr_loss += loss.item()
                 if (step + 1) % args.gradient_accumulation_steps == 0:
                     if args.fp16:
-                        torch.nn.utils.clip_grad_norm_(amp.master_params(optimizer), args.max_grad_norm)
-                    else:
-                        torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
+                        scaler.unscale_(optimizer)
+                    torch.nn.utils.clip_grad_norm_(model.parameters(), args.max_grad_norm)
 
-                    optimizer.step()
+                    if args.fp16:
+                        scaler.step(optimizer)
+                        scaler.update()
+                    else:
+                        optimizer.step()
                     scheduler.step()  # Update learning rate schedule
                     model.zero_grad()
                     global_step += 1
@@ -685,7 +692,7 @@ class Seq2SeqModel:
         Args:
             eval_data: Pandas DataFrame containing the 2 columns - `input_text`, `target_text`.
                         - `input_text`: The input text sequence.
-                        - `target_text`: The target sequence
+                        - `target_text`: The target text sequence.
             output_dir: The directory where model files will be saved. If not given, self.args.output_dir will be used.
             verbose: If verbose, results will be printed to the console on completion of evaluation.
             silent: If silent, tqdm progress bars will be hidden.
