@@ -4,13 +4,12 @@ import pickle
 from multiprocessing import Pool
 from typing import Tuple
 
-from tqdm.auto import tqdm
-
 import pandas as pd
 import torch
 from tokenizers.implementations import ByteLevelBPETokenizer
 from tokenizers.processors import BertProcessing
 from torch.utils.data import Dataset
+from tqdm.auto import tqdm
 from transformers import PreTrainedTokenizer
 
 logger = logging.getLogger(__name__)
@@ -20,11 +19,11 @@ def preprocess_data(data):
     input_text, target_text, encoder_tokenizer, decoder_tokenizer, args = data
 
     input_text = encoder_tokenizer.encode(
-        input_text, max_length=args["max_seq_length"], pad_to_max_length=True, return_tensors="pt",
+        input_text, max_length=args.max_seq_length, pad_to_max_length=True, return_tensors="pt",
     )
 
     target_text = decoder_tokenizer.encode(
-        target_text, max_length=args["max_seq_length"], pad_to_max_length=True, return_tensors="pt"
+        target_text, max_length=args.max_seq_length, pad_to_max_length=True, return_tensors="pt"
     )
     return (torch.flatten(input_text), torch.flatten(target_text))
 
@@ -32,35 +31,35 @@ def preprocess_data(data):
 class Seq2SeqDataset(Dataset):
     def __init__(self, encoder_tokenizer, decoder_tokenizer, args, data, mode):
         cached_features_file = os.path.join(
-            args["cache_dir"], args["model_name"] + "_cached_" + str(args["max_seq_length"]) + str(len(data))
+            args.cache_dir, args.model_name + "_cached_" + str(args.max_seq_length) + str(len(data))
         )
 
         if os.path.exists(cached_features_file) and (
-            (not args["reprocess_input_data"] and not args["no_cache"])
-            or (mode == "dev" and args["use_cached_eval_features"] and not args["no_cache"])
+            (not args.reprocess_input_data and not args.no_cache)
+            or (mode == "dev" and args.use_cached_eval_features and not args.no_cache)
         ):
             logger.info(" Loading features from cached file %s", cached_features_file)
             with open(cached_features_file, "rb") as handle:
                 self.examples = pickle.load(handle)
         else:
-            logger.info(" Creating features from dataset file at %s", args["cache_dir"])
+            logger.info(" Creating features from dataset file at %s", args.cache_dir)
 
         data = [
             (input_text, target_text, encoder_tokenizer, decoder_tokenizer, args)
             for input_text, target_text in zip(data["input_text"], data["target_text"])
         ]
 
-        if args["use_multiprocessing"]:
-            with Pool(args["process_count"]) as p:
+        if args.use_multiprocessing:
+            with Pool(args.process_count) as p:
                 self.examples = list(
                     tqdm(
-                        p.imap(preprocess_data, data, chunksize=args["multiprocessing_chunksize"]),
+                        p.imap(preprocess_data, data, chunksize=args.multiprocessing_chunksize),
                         total=len(data),
-                        disable=args["silent"],
+                        disable=args.silent,
                     )
                 )
         else:
-            self.examples = [preprocess_data(d) for d in tqdm(data, disable=args["silent"])]
+            self.examples = [preprocess_data(d) for d in tqdm(data, disable=args.silent)]
 
         logger.info(" Saving features into cached file %s", cached_features_file)
         with open(cached_features_file, "wb") as handle:
@@ -77,11 +76,11 @@ def preprocess_data_bart(data):
     input_text, target_text, tokenizer, args = data
 
     input_ids = tokenizer.batch_encode_plus(
-        [input_text], max_length=args["max_seq_length"], pad_to_max_length=True, return_tensors="pt",
+        [input_text], max_length=args.max_seq_length, pad_to_max_length=True, return_tensors="pt",
     )
 
     target_ids = tokenizer.batch_encode_plus(
-        [target_text], max_length=args["max_seq_length"], pad_to_max_length=True, return_tensors="pt"
+        [target_text], max_length=args.max_seq_length, pad_to_max_length=True, return_tensors="pt"
     )
 
     return {
@@ -96,35 +95,35 @@ class SimpleSummarizationDataset(Dataset):
         self.tokenizer = tokenizer
 
         cached_features_file = os.path.join(
-            args["cache_dir"], args["model_name"] + "_cached_" + str(args["max_seq_length"]) + str(len(data))
+            args.cache_dir, args.model_name + "_cached_" + str(args.max_seq_length) + str(len(data))
         )
 
         if os.path.exists(cached_features_file) and (
-            (not args["reprocess_input_data"] and not args["no_cache"])
-            or (mode == "dev" and args["use_cached_eval_features"] and not args["no_cache"])
+            (not args.reprocess_input_data and not args.no_cache)
+            or (mode == "dev" and args.use_cached_eval_features and not args.no_cache)
         ):
             logger.info(" Loading features from cached file %s", cached_features_file)
             with open(cached_features_file, "rb") as handle:
                 self.examples = pickle.load(handle)
         else:
-            logger.info(" Creating features from dataset file at %s", args["cache_dir"])
+            logger.info(" Creating features from dataset file at %s", args.cache_dir)
 
         data = [
             (input_text, target_text, tokenizer, args)
             for input_text, target_text in zip(data["input_text"], data["target_text"])
         ]
 
-        if args["use_multiprocessing"]:
-            with Pool(args["process_count"]) as p:
+        if args.use_multiprocessing:
+            with Pool(args.process_count) as p:
                 self.examples = list(
                     tqdm(
-                        p.imap(preprocess_data_bart, data, chunksize=args["multiprocessing_chunksize"]),
+                        p.imap(preprocess_data_bart, data, chunksize=args.multiprocessing_chunksize),
                         total=len(data),
-                        disable=args["silent"],
+                        disable=args.silent,
                     )
                 )
         else:
-            self.examples = [preprocess_data_bart(d) for d in tqdm(data, disable=args["silent"])]
+            self.examples = [preprocess_data_bart(d) for d in tqdm(data, disable=args.silent)]
 
     def __len__(self):
         return len(self.examples)
